package it.openutils.spring.rmibernate.server.exporter;

import it.openutils.spring.rmibernate.server.aspects.HibernateLazyRmiInterceptor;

import org.hibernate.SessionFactory;
import org.springframework.aop.framework.ProxyFactory;
import org.springframework.remoting.rmi.RmiServiceExporter;
import org.springframework.remoting.support.RemoteInvocationTraceInterceptor;
import org.springframework.util.ClassUtils;


/**
 * Rmi exporter that checks laziness of bean fields in all object graph
 * @author mmolaschi
 * @version $Id: $
 */
public class HibernateRmiServiceExporter extends RmiServiceExporter
{

    /**
     * hibernate sessionfactory
     */
    private SessionFactory sessionFactory;

    /**
     * {@inheritDoc}
     */
    @Override
    protected Object getProxyForService()
    {
        checkService();
        checkServiceInterface();
        ProxyFactory proxyFactory = new ProxyFactory();
        proxyFactory.addInterface(getServiceInterface());
        if (isRegisterTraceInterceptor())
        {
            proxyFactory.addAdvice(new RemoteInvocationTraceInterceptor(getExporterName()));
        }
        // add interceptor for lazy work
        proxyFactory.addAdvice(new HibernateLazyRmiInterceptor(sessionFactory));
        proxyFactory.setTarget(getService());
        return proxyFactory.getProxy(ClassUtils.getDefaultClassLoader());
    }

    /**
     * Set hibernate session factory
     * @param sessionFactory hibernate sessionfactory
     */
    public void setSessionFactory(SessionFactory sessionFactory)
    {
        this.sessionFactory = sessionFactory;
    }

}
