package it.openutils.spring.rmibernate.shared;

import it.openutils.spring.rmibernate.client.HibernateRmiProxyFactoryBean;
import it.openutils.spring.rmibernate.client.aspects.HibernateLazyLoaderAspect;

import java.io.ObjectStreamException;
import java.io.Serializable;

import net.sf.cglib.proxy.Enhancer;


/**
 * Store lazy reference
 * @author mmolaschi
 * @version $Id: $
 */
public class LazyReference implements Serializable
{

    /**
     * UID
     */
    private static final long serialVersionUID = 421364413345638150L;

    private Serializable id;

    private String fieldName;

    private String className;

    private String fieldClassName;

    /**
     * default constructor
     */
    public LazyReference()
    {

    }

    /**
     * Constructor
     * @param className className
     * @param fieldName fieldName
     * @param fieldClassName fieldClassName
     * @param id id of current entity
     */
    public LazyReference(String className, String fieldName, String fieldClassName, Serializable id)
    {
        this.className = className;
        this.fieldName = fieldName;
        this.fieldClassName = fieldClassName;
        this.id = id;
    }

    /**
     * Called on deserialization, creates proxy for remote lazy loading
     * @return proxy for remote lazy loading
     * @throws ObjectStreamException exception reading
     * @throws ClassNotFoundException class not found
     */
    @SuppressWarnings("unchecked")
    Object readResolve() throws ObjectStreamException, ClassNotFoundException
    {
        // get proxy superclass
        Class superclass = Class.forName(fieldClassName);
        Class[] interfaces = null;
        // if superclass is an interface add it to interfaces
        if (superclass.isInterface())
        {
            interfaces = new Class[]{superclass};
        }
        // return proxy
        return Enhancer.create(
            superclass,
            interfaces,
            new HibernateLazyLoaderAspect(className, fieldName, id, HibernateRmiProxyFactoryBean.hibernateLazyServiceTL.get()));

    }

    public String getClassName()
    {
        return className;
    }

    public void setClassName(String className)
    {
        this.className = className;
    }

    public Serializable getId()
    {
        return id;
    }

    public void setId(Serializable id)
    {
        this.id = id;
    }

    public String getFieldName()
    {
        return fieldName;
    }

    public void setFieldName(String fieldName)
    {
        this.fieldName = fieldName;
    }


    public String getFieldClassName()
    {
        return fieldClassName;
    }


    public void setFieldClassName(String fieldClassName)
    {
        this.fieldClassName = fieldClassName;
    }

}
